<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class QuestModeloController extends Crud {

    protected $tabelaFolhaService;
    protected $dataSourceName;
    protected $dataSourcePadravaliacao = 'avl_padravaliacao';
    protected $dataSourcePadravaliacaoGet = 'getPadrAvaliacao';
    protected $dataSourceQuestPadraval = 'avl_questpadraval';
    protected $dataSourceQuestPadravalGet = 'getQuestPadrAval';
    protected $dataSourceQuestoesGet = 'getQuestoesQuestionario';
    protected $dataSourceQuestModeloGet = 'getQuestoesModelo';
    protected $dataSourceResposta = 'avl_resposta';
    protected $dataSourceRespostaGet = 'getRespostaPadrAval';
    protected $dataSourceCombinacao = 'avl_combquestpadr';
    protected $dataSourceCombinacaoGet = 'getCombinacao';
    protected $dataSourceAgrupamento = 'avl_agrupquestpadr';
    protected $dataSourceAgrupamentoGet = 'getAgrupamento';
    protected $dataSourceTipoRelac = 'avl_tiporelacpadr';
    protected $dataSourceTipoRelacGet = 'getTipoRelac';
    protected $dataSourceTemaAnalise = 'avl_temaanalise';
    protected $dataSourceTemaAnaliseGet = 'getTemaAnalise';
    protected $dataSourceTipoAval = 'avl_tipoavaliacao_select';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRPADRAVALIACAO', 'P_NRTIPOAVALIACAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, $questModeloService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->questModeloService = $questModeloService;
        parent::__construct($dataSourceManager);
    }

    public function getPadrAvaliacao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourcePadravaliacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);

        $response->addDataSet($dataSet);
    }
    
    public function getQuestPadrAval(DTO\Request\Filter $request, DTO\Response $response) {
        $this->questModeloService->checaCombPad($this->hcmParameters->getNrorgTrabalho());
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceQuestPadravalGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    public function getQuestoesQuestionario(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceQuestoesGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getQuestoesModelo(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceQuestModeloGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getRespostaPadrAval(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceRespostaGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getCombinacao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->questModeloService->checaCombPad($this->hcmParameters->getNrorgTrabalho());
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceCombinacaoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getAgrupamento(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceAgrupamentoGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getTipoRelac(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceTipoRelacGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getTemaAnalise(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceTemaAnaliseGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setTemaAnalise(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceTemaAnalise;
            foreach($request->getDataSet()->getRows() as $row) {
            if (array_key_exists('NRTEMAANALISE', $row)) {
                    if ($row['NRTEMAANALISE'] === '') {
                        $row['NRTEMAANALISE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AVL_TEMAANALISE', 1, 22, true);
                    }
                } else {
                        $row['NRTEMAANALISE'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AVL_TEMAANALISE', 1, 22, true);
                }
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
        
    }
    
     public function deleteTemaAnalise(DTO\Request\DataSet $request, DTO\Response $response){
         try{
            $this->dataSourceName = $this->dataSourceTemaAnalise;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }   
    }
    
    public function setPadrAvaliacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $row["NRNIVELAVALIA"] = isset($row["NRNIVELAVALIA"])?$row["NRNIVELAVALIA"]:null;
                $row["NRCRITERIORESUL"] = isset($row["NRCRITERIORESUL"])?$row["NRCRITERIORESUL"]:null;
                if (array_key_exists('NRPADRAVALIACAO', $row)) {
                    if ($row['NRPADRAVALIACAO'] === '') {
                        $row['NRPADRAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PADRAVALIACAO', 1, 22, true);
                    }
                } else {
                    $row['NRPADRAVALIACAO'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'PADRAVALIACAO', 1, 22, true);
                }
            }
             $this->dataSourceName = $this->dataSourcePadravaliacao;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setQuestPadrAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            
            error_reporting(E_ERROR | E_PARSE);
            $this->dataSourceName = $this->dataSourceQuestPadraval;
            foreach ($request->getDataSet()->getRows() as &$row) {
                
                $row["NRAGRUPQUESTPAD"] = isset($row["NRAGRUPQUESTPAD"])?$row["NRAGRUPQUESTPAD"]:null;
                $row["NRNIVELQUESTAO"] = isset($row["NRNIVELQUESTAO"])?$row["NRNIVELQUESTAO"]:null;
                $row["NRTEMAANALISE"] = isset($row["NRTEMAANALISE"])?$row["NRTEMAANALISE"]:null;
                
                if (array_key_exists('NRQUESTPADRAVAL', $row)){
                    if ($row['NRQUESTPADRAVAL'] === '') {
                        $row['NRQUESTPADRAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTPADRAVAL', 1, 22, true);
                        if(!$this->questModeloService->validaSetQuestao($row['NRORG'],$row['NRPADRAVALIACAO'],$row['NRQUESTAO'])){
                            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Questão já cadastrada para este questionario","E"));
                            return false;
                        }
                    }
                }else{
                    $row['NRQUESTPADRAVAL'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'QUESTPADRAVAL', 1, 22, true);
                    if(!$this->questModeloService->validaSetQuestao($row['NRORG'],$row['NRPADRAVALIACAO'],$row['NRQUESTAO'])){
                            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Questão já cadastrada para este questionario","E"));
                            return false;
                        }
                }
                
                if($row['IDSORTQUESTPADR'] != 'SEM_SORTEIO'){
                    $row['NRQUESTAO'] = $this->questModeloService->getQuestaoSorteio($row['NRORG'],$row['NRNIVELQUESTAO'],$row['IDSORTQUESTPADR'],$row['NRPADRAVALIACAO']);
                    if($row['NRQUESTAO'] == NULL){
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhuma questão encontrada com estes parâmetros","E"));
                    } else
                        parent::save($request, $response);
                } else 
                    parent::save($request, $response);
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setRespostaPadrAval(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRRESPOSTA', $row)) {
                    if ($row['NRRESPOSTA'] === '') {
                        $row['NRRESPOSTA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'RESPOSTA', 1, 22, true);
                    }
                } else {
                    $row['NRRESPOSTA'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'RESPOSTA', 1, 22, true);
                }
            }
            $this->dataSourceName = $this->dataSourceResposta;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setCombinacao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRCOMBQUESTPADR', $row)) {
                    if ($row['NRCOMBQUESTPADR'] === '') {
                        $row['NRCOMBQUESTPADR'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'COMBQUESTPADR', 1, 22, true);
                    }
                } else {
                    $row['NRCOMBQUESTPADR'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'COMBQUESTPADR', 1, 22, true);
                }
            }
            $this->dataSourceName = $this->dataSourceCombinacao;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setAgrupamento(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRAGRUPQUESTPAD', $row)) {
                    if ($row['NRAGRUPQUESTPAD'] === '') {
                        $row['NRAGRUPQUESTPAD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AGRUPQUESTPADR', 1, 22, true);
                    }
                } else {
                    $row['NRAGRUPQUESTPAD'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'AGRUPQUESTPADR', 1, 22, true);
                }
            }
            $this->dataSourceName = $this->dataSourceAgrupamento;
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function setTipoRelac(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRTIPORELACPADR', $row)) {
                    if ($row['NRTIPORELACPADR'] === '') {
                        $row['NRTIPORELACPADR'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPORELACPADR', 1, 22, true);
                    }
                } else {
                    $row['NRTIPORELACPADR'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'TIPORELACPADR', 1, 22, true);
                }
            }
            if($this->questModeloService->relacIsValid($row['NRORG'],$row['NRPADRAVALIACAO'],$row['NRTIPORELACIONA'],$row['__is_new'],$row['NRTIPORELACPADR'])){
            $this->dataSourceName = $this->dataSourceTipoRelac;
            parent::save($request, $response);
            }else{
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Este tipo de relacionamento já existe neste questionário",'E'));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deletePadrAvaliacao(DTO\Request\DataSet $request, DTO\Response $response){
        
        try{
        foreach ($request->getDataSet()->getRows() as &$row) {
            
            $this->dataSourceName = $this->dataSourcePadravaliacao;   
            $questExists = $this->questModeloService->questaoExists($row['NRORG'],$row['NRPADRAVALIACAO']);
            $combExists = $this->questModeloService->agrupExists($row['NRORG'],$row['NRPADRAVALIACAO']);
            $agrupExists = $this->questModeloService->combExists($row['NRORG'],$row['NRPADRAVALIACAO']);
            $relacExists = $this->questModeloService->relacExists($row['NRORG'],$row['NRPADRAVALIACAO']);
            if($questExists || $combExists || $agrupExists || $relacExists){
                $Message = "";
                $exist_array = array($questExists,$combExists,$agrupExists,$relacExists);
                $atr_array = array("Questoes","Combinacoes","Agrupamentos","Relacionamentos");
                $index = 0;
                foreach($exist_array as $exist){
                    if($exist){
                    $Message = " ".$atr_array[$index].",";
                    $index++;
                    }
                }
                $Message = "Existem".$Message."<BR>afiliadas a este questionario,<BR> favor deleta-las antes de exclui-lo";
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($Message,'E'));
            }else{
                parent::delete($request, $response);
            }
        }    
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
        

    }
    
    public function deleteQuestPadrAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceQuestPadraval;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteRespostaPadrAval(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceResposta;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteCombinacao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceCombinacao;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteAgrupamento(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceAgrupamento;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function deleteTipoRelac(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $this->dataSourceName = $this->dataSourceTipoRelac;
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }
    
    public function cadastroAutomaticoResposta(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $dsResposta = $row['DSRESPOSTA'];
                $vrPesoResposta = $row['VRPESORESPOSTA'];
                $nrOrdenaRespost = isset($row['NRORDENARESPOST']) ? $row['NRORDENARESPOST'] : NULL;
                $cdIntegracao = isset($row['CDINTEGRACAO']) ? $row['CDINTEGRACAO'] : NULL;
                $idResposCorreta = $row['IDRESPOSCORRETA'];
                $idAtivo = $row['IDATIVO'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                $nrorgpadrao = $this->hcmParameters->getNrorgPadrao();
                $comb = isset($row['NRCOMBQUESTPADR']) ? $row['NRCOMBQUESTPADR'] : NULL;
                $agrup = isset($row['NRAGRUPQUESTPAD']) ? $row['NRAGRUPQUESTPAD'] : NULL;
                $nivel = isset($row['NRNIVELPADQUEST']) ? $row['NRNIVELPADQUEST'] : NULL;
                $tema = isset($row['NRTEMAANALISE']) ? $row['NRTEMAANALISE'] : NULL;
                $ordem = is_nan($row['NRORDEQUESTPADR']) ? $row['NRORDEQUESTPADR'] : NULL;
                $ordem = isset($row['NRORDEQUESTPADR']) ? $row['NRORDEQUESTPADR'] : NULL;
                $padraval = isset($row['NRPADRAVALIACAO']) ? $row['NRPADRAVALIACAO'] : NULL;
                
                if(!empty($row['NRQUESTAO'])){
                    $nrquestao_list = $row['NRQUESTAO'];
                    $result = $this->questModeloService->cadastroAutomatico($nrquestao_list,$dsResposta,$vrPesoResposta,$nrOrdenaRespost,$cdIntegracao,$idResposCorreta,$idAtivo,$nrorg,$nrorgpadrao,$dtmescompetenc);
                }else{
                    $list = $this->questModeloService->getNrQuestoes($comb,$agrup,$nivel,$tema,$ordem,$nrorg,$padraval);
                    if(!empty($list)){
                        $nrquestao_list =array();
                        foreach ($list as $list_row){
                            $aux = $list_row['NRQUESTAO'];
                            array_push($nrquestao_list,$aux);
                        }
                        $result = $this->questModeloService->cadastroAutomatico($nrquestao_list,$dsResposta,$vrPesoResposta,$nrOrdenaRespost,$cdIntegracao,$idResposCorreta,$idAtivo,$nrorg,$nrorgpadrao,$dtmescompetenc);
                    
                    }else{
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Nenhuma questão encontrada dentro deste filtro",'E' ));
                    }
                }
                $response->addNotification(new \Zeedhi\Framework\DTO\Response\Notification("","success"));
                
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            
        }
    }
    
    public function novaQuestao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as $row) {
                $nrnivelpadquest = $row['NRNIVELPADQUEST'];
                $txquestao       = $row['TXQUESTAO'];
                $idtipoquestao   = $row['IDTIPOQUESTAO'];
                $idquestaoobriga = $row['IDQUESTAOOBRIGA'];
                $idopcaoresposta = $row['IDOPCAORESPOSTA'];
                $dtrevisaquestao = isset($row['DTREVISAQUESTAO']) ? $row['DTREVISAQUESTAO'] : NULL;
                $idativo = $row['IDATIVO'];
                $codigo = array();
                array_push($codigo,$this->questModeloService->novaQuestao($nrnivelpadquest,$txquestao,$idtipoquestao,$dtrevisaquestao,$idquestaoobriga,$idopcaoresposta,$idativo));
            $response->addDataSet(new DataSet("result",$codigo));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 