<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Services\CCTService;
use HCM\FPA\Services\EventoEstruturaService;
use HCM\FPA\Services\ParametroSindicatoService;
use HCM\FPA\Services\TpItemCCTService;
use HCM\Geral\Services\EstruturasService;
use HCM\Geral\Services\ParcnegocioService;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use HCM\VAB\Services\BeneficioService;
use HCM\VAB\Services\TipoBeneficioService;

class ParametrizacaoSindicatoService {

    public function __construct (EntityManager $entityManager, HCMParameters $hcmParameters, ParcnegocioService $parcnegocioService, EstruturasService $estruturasService, 
                                 ParametroSindicatoService $parametroSindicatoService, CCTService $cctService, TpItemCCTService $tpItemCCTService, EventoEstruturaService $eventoEstruturaService, BeneficioService $beneficioService,
                                 TipoBeneficioService $tipoBeneficioService){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->parcnegocioService = $parcnegocioService;
        $this->estruturasService = $estruturasService;
        $this->parametroSindicatoService = $parametroSindicatoService;
        $this->cctService = $cctService;
        $this->tpItemCCTService = $tpItemCCTService;
        $this->eventoEstruturaService = $eventoEstruturaService;
        $this->beneficioService = $beneficioService;
        $this->tipoBeneficioService = $tipoBeneficioService;
    }
    
    public function importarSindicatoPadrao($sindicato) {
        try {
            $this->entityManager->beginTransaction();
            
            if (strlen($sindicato['CDCNPJESTRUT']) == 14) {
                $cdtipoinscricao = 'CNPJ';
            } else if (strlen($sindicato['CDCNPJESTRUT']) == 11) {
                $cdtipoinscricao = 'CPF';
            } else {
                $cdtipoinscricao = 'LIVRE';
            }
            
            //Cria Parceiro de Negócio
            $parceiro = $this->parcnegocioService->novo(
                $this->hcmParameters->getNrorgTrabalho(), $sindicato['NMESTRUTURAH'], $sindicato['NMESTRUTURAH'], 'ESTRUTURA', 'N', 'N',
                $cdtipoinscricao, $sindicato['CDCNPJESTRUT'], DateUtil::getDataDeString($sindicato['DTINIVIGENCIA'], DateUtil::FORMATO_BRASILEIRO_DATAHORA, true)
            );
            
            //Cria Endereço do Parceiro
            $enderecos = $this->entityManager->getRepository(Repositories::ENDERECOPARC)->findBy(array('nrorg' => $sindicato['NRORG'], 'nrparcnegocio' => $sindicato['NRPARCNEGOCIO']));
            foreach($enderecos as $endereco) {
                $this->parcnegocioService->novoEndereco(
                    $parceiro->getNrparcnegocio(), $endereco->getCdpais(), $endereco->getSgestado(), $endereco->getCdmunicipio(), $endereco->getCdlogradouro(), $endereco->getNmbairroendereco(),
                    $endereco->getCdtipoendereco(), $endereco->getNrcependereco(), $endereco->getDscompleendereco(), $endereco->getNrimovelendereco(), $endereco->getDsendereco(),
                    $endereco->getDsreferenciaende()
                );
            }
            
            //Cria Comunicação do Parceiro
            $comunicacoes = $this->entityManager->getRepository(Repositories::COMUNICAPARC)->findBy(array('nrorg' => $sindicato['NRORG'], 'nrparcnegocio' => $sindicato['NRPARCNEGOCIO']));
            foreach($comunicacoes as $comunicacao) {
                $this->parcnegocioService->novaFormacomu(
                    $parceiro->getNrparcnegocio(), $comunicacao->getCdformacomu(), $comunicacao->getCdprefixcomuparc(), $comunicacao->getCdcomunicaparc(), $comunicacao->getCdcomplecomuparc(), $comunicacao->getNrenderecoparc()
                );
            }
            
            //Cria estruturas mestre e histórico
            $estruturas = $this->estruturasService->novo(
                $sindicato['NRTIPOESTRUTURA'], $sindicato['DTINIVIGENCIA'], $sindicato['IDATIVO'], $sindicato['IDOPTSIMPLES'], $parceiro->getNrparcnegocio(), $sindicato['NMESTRUTURAM'], $sindicato['CDCNPJESTRUT'], $sindicato['DTFIMVIGENCIA'], $sindicato['CDINTESTRUTURA'], null, null, $sindicato['CDINTESTRUTEXP'],
                $sindicato['CDINTESTRUTEXPFIL'], null, null, $sindicato['NRTPEVENTOCTBL'], $sindicato['NRREGIMEAPUR'], $sindicato['DTESOCIAL'], $sindicato['NRGRUPOESOCIAL'], $sindicato['DTEXTINCAO'], $sindicato['NRCALENDARIO'], $sindicato['CDCEIESTRUT'], $sindicato['NMRAZSOCESTRUT'],
                $sindicato['CDNATUJURI'], $sindicato['CDCNAE'], $sindicato['NRCAGED'], $sindicato['CDEMPRCAIXA'], $sindicato['IDTIPOEMPR'], $sindicato['IDPARTICIPAT'], $sindicato['NRPROPRIEMPR'], $sindicato['NRPARCRESPINF'], $sindicato['NRPARCENTISIN'], $sindicato['CDFPAS'], $sindicato['CDREDUCONT'], $sindicato['NMFANTASIA'], 
                $sindicato['NMESTRUTURAH'], $sindicato['CDCPFESTRUTURA'], DateUtil::getDataDeString($sindicato['DTBASESINDICAL'], DateUtil::FORMATO_BRASILEIRO_DATAHORA, true), $sindicato['CDSINDICAL'], $sindicato['CDTERCEIRO'], $sindicato['NRCLASSTRIBUT'], $sindicato['VRFAP'], $sindicato['NRTIPOTRIBUT'], $sindicato['VRALIQDESON'], $sindicato['CDFEDERALESTRUT'], $sindicato['NRPARCREGAMB'], 
                $sindicato['NRPARCMONBIO'], $sindicato['DTINIRESAMB'], $sindicato['DTFIMRESAMB'], $sindicato['NRTPOBRIGAPRENDIZ'], $sindicato['NRTPOBRIGAPCD'], $sindicato['CDCAEPFESTRUT'], $sindicato['CODCONVENIO']
            );
            
            //Cria Parametro de Sindicato
            $parametros = $this->entityManager->getRepository(Repositories::FPA_PARAMSINDICATO)->findBy(array('nrorg' => $sindicato['NRORG'], 'nrestruturam' => $sindicato['NRESTRUTURAM']));
            foreach($parametros as $parametro) {
                $paramsindicato = $this->parametroSindicatoService->setParametroSindicato(
                    $this->hcmParameters->getNrorgTrabalho(), $estruturas[0]->getNrestruturam(), 'S', $parametro->getIdultrperantfer(), $parametro->getDtiniciovigencia(), $parametro->getDtfimvigencia(), $parametro->getIdultranoant13(), 
                    $parametro->getNrmesesmedia1(), $parametro->getNrmesesmedia2(), $parametro->getNrmesesmedia3(), $parametro->getCdsindical(), $parametro->getDtbasesindical(), $parametro->getNrdiv13antecip(), $parametro->getNrdiasapurvales(), $parametro->getVrperdescvale()
                );
            }
            
            //Cria CCT
            //Cria CCT - Item
            //Cria CCT - Benefício
            $ccts = $this->entityManager->getRepository(Repositories::FPA_CADASTROCCT)->findBy(array('nrorg' => $sindicato['NRORG'], 'nrestruturam' => $sindicato['NRESTRUTURAM']));
            foreach($ccts as $cct) {
                $novoCCT = $this->cctService->novoCCT($estruturas[0]->getNrestruturam(), $cct->getDtanobasecct(), $cct->getDtinivigenciacct(), $cct->getDtfinvigenciacct());
                
                $itensCCT = $this->entityManager->getRepository(Repositories::FPA_ITCADASTROCCT)->findBy(array('nrorg' => $cct->getNrorg(), 'nrcadastrocct' => $cct->getNrcadastrocct()));
                foreach($itensCCT as $itemCCT) {
                
                    $tipoitemcctPadrao = $this->entityManager->getRepository(Repositories::GPE_TIPOITEMCCT)->findOneBy(array('nrorg' => $itemCCT->getNrorg(), 'nrtipoitemcct' => $itemCCT->getNrtipoitemcct()));
                    
                    $tipoitemcct = $this->entityManager->getRepository(Repositories::GPE_TIPOITEMCCT)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'nmtipoitemcct' => $tipoitemcctPadrao->getNmtipoitemcct()));
                    if (!is_object($tipoitemcct)) {
                        $tipoitemcct = $this->tpItemCCTService->novo($tipoitemcctPadrao->getNmtipoitemcct());
                    }
                    
                    $this->cctService->novoItemCCT(
                        $novoCCT->getNrcadastrocct(), $tipoitemcct->getNrtipoitemcct(), $itemCCT->getDsitemcct(), $itemCCT->getDslegislacaoitcct(), $itemCCT->getNrocupacaom(), $itemCCT->getNrescalatrabm(), $itemCCT->getNrsitufuncm(), $itemCCT->getNrtpdemissao(), $itemCCT->getNreventom(), $itemCCT->getNrfaixatabfolha(), $itemCCT->getNrvalorreferenciam(),
                        $itemCCT->getVrpisoitcct(), $itemCCT->getVrtetoitcct(), $itemCCT->getDtiniitemcct(), $itemCCT->getDtfinitemcct(), $itemCCT->getVrpercentualitcct(), $itemCCT->getVritemcct(), $itemCCT->getDsiniintervalocct(), $itemCCT->getDsfinintervalocct(), $itemCCT->getVridademinimacct(), $itemCCT->getVridademaximacct(),
                        $itemCCT->getQtdemesesitcct(), $itemCCT->getQtdeanomincct(), $itemCCT->getQtdeanomaxcct(), $itemCCT->getQtdiasestabitcct()
                    );
                }
                
                $beneficiosCCT = $this->entityManager->getRepository(Repositories::VAB_BENEFICIOCCT)->findBy(array('nrorg' => $cct->getNrorg(), 'nrcadastrocct' => $cct->getNrcadastrocct()));
                foreach($beneficiosCCT as $beneficioCCT) {
                
                    $tpbeneficioPadrao = $this->entityManager->getRepository(Repositories::TPBENEFICIO)->findOneBy(array('nrorg' => $beneficioCCT->getNrorg(), 'nrtpbeneficio' => $beneficioCCT->getNrtpbeneficio()));
                    
                    $tpbeneficio = $this->entityManager->getRepository(Repositories::TPBENEFICIO)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dstpbeneficio' => $tpbeneficioPadrao->getDstpbeneficio()));
                    if (!is_object($tpbeneficio)) {
                        $tpbeneficio = $this->tipoBeneficioService->novo($tpbeneficioPadrao->getDstpbeneficio());
                    }
                    
                    $this->cctService->novoBeneficioCCT(
                        $novoCCT->getNrcadastrocct(), $tpbeneficio->getNrtpbeneficio(), $beneficioCCT->getNridademaxsubfilhos(), $beneficioCCT->getSexosubsidiofilho(), $beneficioCCT->getNridademaxsubdep(), $beneficioCCT->getSexosubsidiodep(), $beneficioCCT->getNrevetitularvinc(), $beneficioCCT->getNrtipoparticipatitular(), $beneficioCCT->getNrtabfaixasaltit(), $beneficioCCT->getVrtitularsind(), $beneficioCCT->getVrtetotitularsind(),
                        $beneficioCCT->getVrpisotitularsind(), $beneficioCCT->getNrevedepevinc(), $beneficioCCT->getNrtipoparticipadep(), $beneficioCCT->getNrtabfaixasaldep(), $beneficioCCT->getVrdepsind(), $beneficioCCT->getVrpisodepsind(), $beneficioCCT->getVrtetodepsind(), $beneficioCCT->getIdativo(), $beneficioCCT->getNreventomensa(), $beneficioCCT->getNreventomensadep(), $beneficioCCT->getVrtetotitularmens(), $beneficioCCT->getVrpisotitularmens(), 
                        $beneficioCCT->getVrlinearbeneficio(), $beneficioCCT->getVrlinearbeneficiodep(), $beneficioCCT->getIdpropadmis(), $beneficioCCT->getIdpropresc(), $beneficioCCT->getQtdiasminmes(), $beneficioCCT->getQtdiasmaxfaltas(), $beneficioCCT->getQtdiasmaxatest(), $beneficioCCT->getQtmesmaxafast(), $beneficioCCT->getVrmultiplicador(), $beneficioCCT->getDsvinculoempreg(), $beneficioCCT->getDsincidevento(), $beneficioCCT->getVrfixacresdesc(), 
                        $beneficioCCT->getVrperacrescdesc(), $beneficioCCT->getIdnaogerbenvincexp(), $beneficioCCT->getIdnaogerabenperexp(), $beneficioCCT->getIdnaogerabenresc(), $beneficioCCT->getVrparticipatitular(), $beneficioCCT->getVrpisotitular(), $beneficioCCT->getVrtetotitular(), $beneficioCCT->getVrparticipadepe(), $beneficioCCT->getVrpisodep(), $beneficioCCT->getVrtetodep(), $beneficioCCT->getQtmaxhsreduzida(), $beneficioCCT->getVrhsreduzida(), 
                        $beneficioCCT->getDsincidevenperder(), $beneficioCCT->getQtmesretrocmult(), $beneficioCCT->getQtmesretrocsemben(), $beneficioCCT->getDsafastamento1(), $beneficioCCT->getIdgeradescafast1(), $beneficioCCT->getDsafastamento2(), $beneficioCCT->getQtmesmaxafast2(), $beneficioCCT->getIdgeradescafast2(), $beneficioCCT->getDsafastamento3()
                    );
                }
            }
            
            //Cria Evento Relacionado
            $eventos = $this->entityManager->getRepository(Repositories::FPA_ESTRUTEVENTO)->findBy(array('nrorg' => $sindicato['NRORG'], 'nrestruturam' => $sindicato['NRESTRUTURAM']));
            foreach($eventos as $evento) {
                $eventoEstrut = $this->eventoEstruturaService->novo(
                    $estruturas[0]->getNrestruturam(), $evento->getNreventom(), $evento->getDtinivigencia(), $evento->getDtfimvigencia(), $evento->getPeriodicidademes(), $evento->getNrvalorreferenciam(),
                    $evento->getVrpercent(), $evento->getVrmultiplic(), $evento->getVrdivide(), $evento->getVreventofixo(), $evento->getIdrefgerar(), $evento->getIdretreferencia(), $evento->getVrreffixo(), 
                    $evento->getIdtipomovimento(), $evento->getVrpisoevento(), $evento->getVrtetoevento(), $evento->getDsobservacao(), $evento->getNrtpagrupacalc(), $evento->getIdgeraafastado()
                );
            }
            
            //Cria Benefício
            $beneficios = $this->entityManager->getRepository(Repositories::VAB_BENEFICIOM)->retornaBeneficiosSindicato($sindicato['NRORG'], $this->hcmParameters->getDtCompetencia(), $sindicato['NRESTRUTURAM']);
            foreach($beneficios as $beneficio) {
                
                $tpbeneficioPadrao = $this->entityManager->getRepository(Repositories::TPBENEFICIO)->findOneBy(array('nrorg' => $beneficio->getNrorg(), 'nrtpbeneficio' => $beneficio->historico->getNrtpbeneficio()));
                
                $tpbeneficio = $this->entityManager->getRepository(Repositories::TPBENEFICIO)->findOneBy(array('nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dstpbeneficio' => $tpbeneficioPadrao->getDstpbeneficio()));
                if (!is_object($tpbeneficio)) {
                    $tpbeneficio = $this->tipobeneficioService->novo($tpbeneficioPadrao->getDstpbeneficio());
                }
                
                $estrutPadrao = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstrutura($beneficio->getNrorg(), $beneficio->historico->getNrestruturam(), $this->hcmParameters->getDtCompetencia());
                
                $estrutBeneficio = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->getEstruturaPorCnpj($this->hcmParameters->getNrorgTrabalho(), $estrutPadrao->historico->getCdcnpjestrut(), $this->hcmParameters->getDtCompetencia());
                if (!is_object($estrutBeneficio)) {
                    $parcnegocio = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrorg' => $estrutPadrao->getNrorg(), 'nrparcnegocio' => $estrutPadrao->historico->getNrparcnegocio()));
                    
                    //Cria Parceiro de Negócio
                    $parceiro = $this->parcnegocioService->novo(
                        $this->hcmParameters->getNrorgTrabalho(), $parcnegocio->getNmprincipalparc(), $parcnegocio->getNmsecundariparc(), $parcnegocio->getCdtipoparcprincipal(), $parcnegocio->getIdpessoafisica(), 
                        $parcnegocio->getIdinstituicao(), $parcnegocio->getCdtipoinscricao(), $parcnegocio->getNrinscricaoparc(), $parcnegocio->getDtnascifundparc()
                    );
                    
                    //Cria estruturas mestre e histórico
                    $estruturas = $this->estruturasService->novo(
                        $estrutPadrao->getNrtipoestrutura(), $estrutPadrao->getDtinivigencia(), $estrutPadrao->historico->getIdativo(), $estrutPadrao->historico->getIdoptsimples(), $parceiro->getNrparcnegocio(), $estrutPadrao->getNmestruturam(), $estrutPadrao->historico->getCdcnpjestrut(), $estrutPadrao->getDtfimvigencia(), $estrutPadrao->getCdintestrutura(), $estrutPadrao->getNrestruturamold(), $estrutPadrao->historico->getNmestruturah(), $estrutPadrao->getCdintestrutexp(),
                        $estrutPadrao->getCdintestrutexpfil(), null, null, $estrutPadrao->getNrtpeventoctbl(), null, null, null, $estrutPadrao->historico->getDtextincao(), $estrutPadrao->historico->getNrcalendario(), $estrutPadrao->historico->getCdceiestrut(), $estrutPadrao->historico->getNmrazsocestrut(),
                        $estrutPadrao->historico->getCdnatujuri(), $estrutPadrao->historico->getCdcnae(), $estrutPadrao->historico->getNrcaged(), $estrutPadrao->historico->getCdemprcaixa(), $estrutPadrao->historico->getIdtipoempr(), $estrutPadrao->historico->getIdparticipat(), $estrutPadrao->historico->getNrpropriempr(), $estrutPadrao->historico->getNrparcrespinf(), $estrutPadrao->historico->getNrparcentisin(), $estrutPadrao->historico->getCdfpas(), $estrutPadrao->historico->getCdreducont(), $estrutPadrao->historico->getNmfantasia(), 
                        $estrutPadrao->historico->getNmestruturah(), $estrutPadrao->historico->getCdcpfestrutura(), $estrutPadrao->historico->getDtbasesindical(), $estrutPadrao->historico->getCdsindical(), $estrutPadrao->historico->getCdterceiro(), $estrutPadrao->historico->getNrclasstribut(), $estrutPadrao->historico->getVrfap(), $estrutPadrao->historico->getNrtipotribut(), $estrutPadrao->historico->getVraliqdeson(), $estrutPadrao->historico->getCdfederalestrut(), $estrutPadrao->historico->getNrparcregamb(), 
                        $estrutPadrao->historico->getNrparcmonbio(), $estrutPadrao->historico->getDtiniresamb(), $estrutPadrao->historico->getDtfimresamb(), null, null, $estrutPadrao->historico->getCdcaepfestrut(), null
                    );
                    
                    $estrutBeneficio = $estruturas[0];
                    $estrutBeneficio->historico = $estruturas[1];
                }
                
                $beneficiom = $this->beneficioService->novoMestre($beneficio->getDtiniciovigencia(), $beneficio->getDtfinalvigencia(), $beneficio->getIdativo());
                
                $beneficioh = $this->beneficioService->novoHistorico(
                    $beneficiom->getNrbeneficiom(), $tpbeneficio->getNrtpbeneficio(), $beneficio->historico->getNrformabeneficio(), $estrutBeneficio->getNrestruturam(), $beneficio->historico->getDtmescompetenc(), $beneficio->historico->getDtreferencia(), $beneficio->historico->getDsbeneficio(), $beneficio->historico->getTpperiodobenef(), $beneficio->historico->getDsobsbeneficio(), $beneficio->historico->getNrtabelafolhapri(), $beneficio->historico->getNrtabelafolhasec(), $beneficio->historico->getIdativo(), $beneficio->historico->getNreventomensa(), 
                    $beneficio->historico->getNrevetitularemp(), $beneficio->historico->getNrevedepempresa(), $beneficio->historico->getNrevetitularvinc(), $beneficio->historico->getNrevedepevinc(), $beneficio->historico->getNrtabfaixasaltit(), $beneficio->historico->getNrtabfaixasaldep(), $beneficio->historico->getNrmesaniversario(), $beneficio->historico->getNrdiavencboleto(), $beneficio->historico->getVrlinearbeneficio(), $beneficio->historico->getNridademaxsubfilhos(), $beneficio->historico->getNridademaxsubdep(), 
                    $beneficio->historico->getNrtipoparticipatitular(), $beneficio->historico->getVrparticipatitular(), $beneficio->historico->getNrtipoparticipadep(), $beneficio->historico->getVrparticipadepe(), $beneficio->historico->getVrlinearbeneficiodep(), $beneficio->historico->getNreventomensadep(), $beneficio->historico->getVrpisotitular(), $beneficio->historico->getVrpisodep(), $beneficio->historico->getVrtetodep(), $beneficio->historico->getVrtetotitular(), $beneficio->historico->getSexosubsidiofilho(), 
                    $beneficio->historico->getSexosubsidiodep(), $beneficio->historico->getIdconsfaltabenef(), $beneficio->historico->getQtmesesfaltabenef(), $beneficio->historico->getIdconsatestbenef(), $beneficio->historico->getQtmesesatestbenef(), $beneficio->historico->getIdconsafastbenef(), $beneficio->historico->getQtmesesafastbenef(), $beneficio->historico->getIdconsferiasbenef(), $beneficio->historico->getQtmesesferiasbenef(), $beneficio->historico->getIdconsferiadobenef(), 
                    $beneficio->historico->getQtmesesferiadobenef(), $beneficio->historico->getIdconsdsrbenef(), $beneficio->historico->getQtmesesdsrbenef(), $beneficio->historico->getIdgerabeneffalta(), $beneficio->historico->getIdgerabenefatest(), $beneficio->historico->getIdgerabenefafast(), $beneficio->historico->getIdgerabenefferias(), $beneficio->historico->getConsvincresmesant(), $estruturas[0]->getNrestruturam(), $beneficio->historico->getDtdisponib(), $beneficio->historico->getIdpropadmis(), 
                    $beneficio->historico->getIdpropresc(), $beneficio->historico->getNreventocopart(), $beneficio->historico->getNreventocopartdep(), $beneficio->historico->getPercoparticipatitu(), $beneficio->historico->getPercoparticipadepe(), $beneficio->historico->getVrdepsind(), $beneficio->historico->getVrtitularsind(), $beneficio->historico->getIdpropdtinscr(), $beneficio->historico->getIdpropdtcanc(), $beneficio->historico->getIdduplicmens(), $beneficio->historico->getNrdiarescisao(), $beneficio->historico->getNreventotaxa(), 
                    $beneficio->historico->getVreventotaxa(), $beneficio->historico->getVrtetotitularsind(), $beneficio->historico->getVrpisotitularsind(), $beneficio->historico->getVrpisodepsind(), $beneficio->historico->getVrtetodepsind(), $beneficio->historico->getVrtetotitularmens(), $beneficio->historico->getVrpisotitularmens(), $beneficio->historico->getIdgeradescafast(), $beneficio->historico->getPeriodicidademes(), $beneficio->historico->getIdgerbenefadmissao(), $beneficio->historico->getIdgerbenefrescisao()
                );
            }
            
            $this->entityManager->commit();
        } catch(\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }
} 