<?php
namespace HCM\VAB\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Folha\Entities\FpaItecalcfolha;
use HCM\Util\DateUtil;

class HistbenefvincRepository {

    protected  $connection;

    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
    }

    public function getVinculosLiquidoBeneficio($nrorg, $dtmescompetenc, $nrcalcfolha, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom, $listVincLiquido)
    {
        if(!($dtmescompetenc instanceof \DateTime)){
            $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
        }

        $dtiniperiodo = $dtmescompetenc->format('d/m/Y');
        $dtfimperiodo = $dtmescompetenc->format('t/m/Y');

        $condTpBeneficio = null;
        $condEstrutura = null;

        if($nrtpbeneficio){
            $condTpBeneficio = "INNER JOIN VAB_BENEFICIOH
    ON VAB_BENEFICIOH.NRORG = :P_NRORG
   AND (VAB_BENEFICIOH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
   AND VAB_BENEFICIOH.TPPERIODOBENEF = 'DIARIO'
   AND VAB_BENEFICIOH.NRBENEFICIOM = VAB_BENEFICIOM.NRBENEFICIOM
   AND VAB_BENEFICIOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH H1
                                         WHERE NRORG = VAB_BENEFICIOH.NRORG
                                           AND NRBENEFICIOM = VAB_BENEFICIOH.NRBENEFICIOM
                                           AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        if($nrestrutlegal || $nrestrutgeren){
            $condEstrutura = "INNER JOIN GPE_VINCULOH
    ON GPE_VINCULOH.NRORG = :P_NRORG
   AND (GPE_VINCULOH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
   AND (GPE_VINCULOH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
   AND GPE_VINCULOH.NRVINCULOM = VAB_HISTBENEFVINC.NRVINCULOM
   AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                       WHERE NRORG = GPE_VINCULOH.NRORG
                                         AND NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                         AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        $sql = <<<SQL
SELECT
    VAB_HISTBENEFVINC.NRVINCULOM,
    VAB_HISTBENEFVINC.NRCALCULOFOLHA,
    VAB_HISTBENEFVINC.NRORG,
    TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC,'DD/MM/YYYY') DTMESCOMPETENC,
    TO_CHAR(VAB_HISTBENEFVINC.DTPGTOBENEF, 'DD/MM/YYYY') DTPGTOBENEF,
    MAX(VAB_HISTBENEFVINC.QTDIASBENEF) QTDIASBENEF,
    0 /*SUM(VAB_HISTBENEFVINC.VRTOTALBENEF)*/ VREVENTO
  FROM VAB_HISTBENEFVINC
 INNER JOIN VAB_BENEFICIOM
    ON VAB_BENEFICIOM.NRORG = :P_NRORG
   AND VAB_BENEFICIOM.NRBENEFICIOM = VAB_HISTBENEFVINC.NRBENEFICIOM
   AND VAB_BENEFICIOM.DTINICIOVIGENCIA <= '$dtiniperiodo'
   AND (VAB_BENEFICIOM.DTFINALVIGENCIA >= '$dtfimperiodo' OR VAB_BENEFICIOM.DTFINALVIGENCIA IS NULL)
 $condTpBeneficio
 $condEstrutura
 WHERE VAB_HISTBENEFVINC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
   AND VAB_HISTBENEFVINC.NRORG = :P_NRORG
   AND TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC, 'DD/MM/YYYY') = :P_DTMESCOMPETENC
   AND VAB_HISTBENEFVINC.NRVINCULOM NOT IN ($listVincLiquido)
   AND (INSTR(:P_LISTA_NRVINCULOM, '_'||VAB_HISTBENEFVINC.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
   AND (VAB_HISTBENEFVINC.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM is null)
 GROUP BY VAB_HISTBENEFVINC.NRVINCULOM,
    VAB_HISTBENEFVINC.NRCALCULOFOLHA,
    VAB_HISTBENEFVINC.NRORG,
    TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC,'DD/MM/YYYY'),
    TO_CHAR(VAB_HISTBENEFVINC.DTPGTOBENEF, 'DD/MM/YYYY')
SQL;

        $param = array(
            'P_NRORG'            => $nrorg,
            'P_NRCALCULOFOLHA'   => $nrcalcfolha,
            'P_DTMESCOMPETENC'   => $dtmescompetenc->format('d/m/Y'),
            'P_NRBENEFICIOM'     => $nrbeneficiom,
            'P_NRTPBENEFICIO'    => $nrtpbeneficio,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
        );

        $result = $this->connection->fetchAll($sql, $param);

        return $result;
    }

    public function getVinculosBeneficio($nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom)
    {
        if(!($dtmescompetenc instanceof \DateTime)){
            $dtmescompetenc = DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true);
        }

        $dtiniperiodo = $dtmescompetenc->format('d/m/Y');
        $dtfimperiodo = $dtmescompetenc->format('t/m/Y');

        $condTpBeneficio = null;
        $condEstrutura = null;

        if($nrtpbeneficio){
            $condTpBeneficio = "INNER JOIN VAB_BENEFICIOH
    ON VAB_BENEFICIOH.NRORG = :P_NRORG
   AND (VAB_BENEFICIOH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
   AND VAB_BENEFICIOH.TPPERIODOBENEF = 'DIARIO'
   AND VAB_BENEFICIOH.NRBENEFICIOM = VAB_BENEFICIOM.NRBENEFICIOM
   AND VAB_BENEFICIOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH H1
                                         WHERE NRORG = VAB_BENEFICIOH.NRORG
                                           AND NRBENEFICIOM = VAB_BENEFICIOH.NRBENEFICIOM
                                           AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        if($nrestrutlegal || $nrestrutgeren){
            $condEstrutura = "INNER JOIN GPE_VINCULOH
    ON GPE_VINCULOH.NRORG = :P_NRORG
   AND (GPE_VINCULOH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
   AND (GPE_VINCULOH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
   AND GPE_VINCULOH.NRVINCULOM = VAB_HISTBENEFVINC.NRVINCULOM
   AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                       WHERE NRORG = GPE_VINCULOH.NRORG
                                         AND NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                         AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        $sql = <<<SQL
SELECT
    VAB_HISTBENEFVINC.NRHISTBENEFVINC,
    VAB_HISTBENEFVINC.NRVINCULOM,
    VAB_HISTBENEFVINC.NRCALCULOFOLHA,
    VAB_HISTBENEFVINC.NREVENTOM,
    VAB_HISTBENEFVINC.NRORG,
    TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC,'DD/MM/YYYY') DTMESCOMPETENC,
    TO_CHAR(VAB_HISTBENEFVINC.DTPGTOBENEF, 'DD/MM/YYYY') DTPGTOBENEF,
    MAX(VAB_HISTBENEFVINC.QTDIASBENEF) QTDIASBENEF,
    SUM(VAB_HISTBENEFVINC.VRTOTALBENEF) VREVENTO
  FROM VAB_HISTBENEFVINC
 INNER JOIN VAB_BENEFICIOM
    ON VAB_BENEFICIOM.NRORG = :P_NRORG
   AND VAB_BENEFICIOM.NRBENEFICIOM = VAB_HISTBENEFVINC.NRBENEFICIOM
   AND VAB_BENEFICIOM.DTINICIOVIGENCIA <= '$dtiniperiodo'
   AND (VAB_BENEFICIOM.DTFINALVIGENCIA >= '$dtfimperiodo' OR VAB_BENEFICIOM.DTFINALVIGENCIA IS NULL)
 $condTpBeneficio
 $condEstrutura
 WHERE VAB_HISTBENEFVINC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
   AND VAB_HISTBENEFVINC.NRORG = :P_NRORG
   AND TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC, 'DD/MM/YYYY') = :P_DTMESCOMPETENC
   AND (INSTR(:P_LISTA_NRVINCULOM, '_'||VAB_HISTBENEFVINC.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
   AND (VAB_HISTBENEFVINC.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM is null)
 GROUP BY VAB_HISTBENEFVINC.NRHISTBENEFVINC,
    VAB_HISTBENEFVINC.NRVINCULOM,
    VAB_HISTBENEFVINC.NRCALCULOFOLHA,
    VAB_HISTBENEFVINC.NREVENTOM,
    VAB_HISTBENEFVINC.NRORG,
    TO_CHAR(VAB_HISTBENEFVINC.DTMESCOMPETENC,'DD/MM/YYYY'),
    TO_CHAR(VAB_HISTBENEFVINC.DTPGTOBENEF, 'DD/MM/YYYY')
SQL;

        $param = array(
            'P_NRORG'            => $nrorg,
            'P_NRCALCULOFOLHA'   => $nrcalculobenef,
            'P_DTMESCOMPETENC'   => $dtmescompetenc->format('d/m/Y'),
            'P_NRBENEFICIOM'     => $nrbeneficiom,
            'P_NRTPBENEFICIO'    => $nrtpbeneficio,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
        );

        $result = $this->connection->fetchAll($sql, $param);

        return $result;
    }

    public function getHistoricosItens($nrorg, $dtmescomepetencia, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom)
    {
        if(!($dtmescomepetencia instanceof \DateTime)){
            $dtmescomepetencia = DateUtil::getDataDeString($dtmescomepetencia, DateUtil::FORMATO_BRASILEIRO, true);
        }

        $dtiniperiodo = $dtmescomepetencia->format('d/m/Y');
        $dtfimperiodo = $dtmescomepetencia->format('t/m/Y');

        $condTpBeneficio = null;
        $condEstrutura = null;

        if($nrtpbeneficio){
            $condTpBeneficio = "JOIN VAB_BENEFICIOH
    ON VAB_BENEFICIOH.NRORG = :P_NRORG
   AND (VAB_BENEFICIOH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
   AND VAB_BENEFICIOH.TPPERIODOBENEF = 'DIARIO'
   AND VAB_BENEFICIOH.NRBENEFICIOM = VAB_BENEFICIOM.NRBENEFICIOM
   AND VAB_BENEFICIOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH H1
                                         WHERE NRORG = VAB_BENEFICIOH.NRORG
                                           AND NRBENEFICIOM = VAB_BENEFICIOH.NRBENEFICIOM
                                           AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        if($nrestrutlegal || $nrestrutgeren){
            $condEstrutura = "JOIN GPE_VINCULOH
    ON GPE_VINCULOH.NRORG = :P_NRORG
   AND (GPE_VINCULOH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
   AND (GPE_VINCULOH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
   AND GPE_VINCULOH.NRVINCULOM = HB.NRVINCULOM
   AND GPE_VINCULOH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                       WHERE NRORG = GPE_VINCULOH.NRORG
                                         AND NRVINCULOM = GPE_VINCULOH.NRVINCULOM
                                         AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
        }

        $sql = <<<SQL
SELECT
      IT.NRVINCULOM,
      IT.NREVENTOM,
      IT.VRREFFOLHA AS VRREFFOLHALOTE,
      IT.VREVENTOFOLHA AS VREVENTOLOTE
  FROM FPA_ITECALCFOLHA IT
  JOIN VAB_HISTBENEFVINC HB ON HB.NRORG = :P_NRORG
                           AND HB.DTMESCOMPETENC = :P_DTMESCOMPETENC
                           AND HB.NRVINCULOM = IT.NRVINCULOM
                           AND HB.NREVENTOM = IT.NREVENTOM
                           AND (HB.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM is null)
                           AND (INSTR(:P_LISTA_NRVINCULOM, '_'||HB.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
  JOIN VAB_BENEFICIOM
    ON VAB_BENEFICIOM.NRORG = :P_NRORG
   AND VAB_BENEFICIOM.NRBENEFICIOM = HB.NRBENEFICIOM
   AND VAB_BENEFICIOM.DTINICIOVIGENCIA <= '$dtiniperiodo'
   AND (VAB_BENEFICIOM.DTFINALVIGENCIA >= '$dtfimperiodo' OR VAB_BENEFICIOM.DTFINALVIGENCIA IS NULL)
  $condTpBeneficio
  $condEstrutura
 WHERE IT.NRORG = :P_NRORG
   AND IT.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
   AND IT.NREVENTOM NOT IN (3030,3040)
SQL;

        $param = array(
            'P_NRORG'            => $nrorg,
            'P_NRCALCULOFOLHA'   => $nrcalculobenef,
            'P_DTMESCOMPETENC'   => $dtmescomepetencia->format('d/m/Y'),
            'P_NRBENEFICIOM'     => $nrbeneficiom,
            'P_NRTPBENEFICIO'    => $nrtpbeneficio,
            'P_NRESTRUTLEGAL'    => $nrestrutlegal,
            'P_NRESTRUTGEREN'    => $nrestrutgeren,
            'P_LISTA_NRVINCULOM' => $lista_nrvinculom,
        );

        $result = $this->connection->fetchAll($sql, $param);

        return $result;
    }

    /**
     * Retorna os itens de cálculo da folha de acordo com os eventos dos históricos de benefícios e filtro informados
     * @param $nrorg                Organização
     * @param $dtmescompetenc       Competência (string)
     * @param $nrcalculomensal      Número de calculo de benefício
     * @param $nrbeneficiom         Número do benefício do histórico (opcional)
     * @param $nrtpbeneficio        Número do tipo do benefício do histórico (opcional)
     * @param $nrestrutlegal        Estrutura legal do vínculo do histórico (opcional)
     * @param $nrestrutgeren        Estrutura gerencial do vínculo do histórico (opcional)
     * @param $lista_nrvinculom     Lista de vínculos - Formato: '_#####_#####_' (opcional)
     * @return array FpaItecalcfolha
     * @throws \Exception
     */
    public function getItensFolhaPorEventoHistorico($nrorg, $dtmescompetenc, $nrcalculomensal, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom) {
        try {
            $params = array(
                'P_NRORG'            => $nrorg,
                'P_NRCALCULOFOLHA'   => $nrcalculomensal,
                'P_DTMESCOMPETENC'   => $dtmescompetenc,
                'P_NRBENEFICIOM'     => $nrbeneficiom,
                'P_NRTPBENEFICIO'    => $nrtpbeneficio,
                'P_NRESTRUTLEGAL'    => $nrestrutlegal,
                'P_NRESTRUTGEREN'    => $nrestrutgeren,
                'P_LISTA_NRVINCULOM' => $lista_nrvinculom
            );

            $condTpBeneficio = null;
            $condEstrutura = null;

            if($nrtpbeneficio){
                $condTpBeneficio = "JOIN VAB_BENEFICIOH BH ON BH.NRORG = :P_NRORG
                                                                      AND (BH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
                                                                      AND BH.TPPERIODOBENEF = 'DIARIO'
                                                                      AND BH.NRBENEFICIOM = HB.NRBENEFICIOM
                                                                      AND BH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH
                                                                                                WHERE NRORG = BH.NRORG
                                                                                                  AND NRBENEFICIOM = BH.NRBENEFICIOM
                                                                                                  AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
            }

            if($nrestrutlegal || $nrestrutgeren){
                $condEstrutura = "JOIN GPE_VINCULOH VH ON VH.NRORG = :P_NRORG
                                                                      AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
                                                                      AND (VH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
                                                                      AND VH.NRVINCULOM = HB.NRVINCULOM
                                                                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                                                                WHERE NRORG = VH.NRORG
                                                                                                  AND NRVINCULOM = VH.NRVINCULOM
                                                                                                  AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
            }

            $where = "NRORG = :P_NRORG
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                AND (NRVINCULOM, NREVENTOM) IN (SELECT HB.NRVINCULOM, HB.NREVENTOM
                                                  FROM VAB_HISTBENEFVINC HB
                                                  $condTpBeneficio
                                                  $condEstrutura
                                                 WHERE HB.NRORG = :P_NRORG
                                                   AND (HB.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM IS NULL)
                                                   AND (INSTR(:P_LISTA_NRVINCULOM, '_'||HB.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A'))";
            $itens = FpaItecalcfolha::getObjectsByWhere($where, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os itens de cálculo dos históricos de benefícios conforme o evento e filtro informados
     * @param $nreventom            Evento a ser retornado ou nulo para todos eventos
     * @param $nrorg                Organização
     * @param $dtmescompetenc       Competência (string)
     * @param $nrcalculobenef       Número de calculo de benefício
     * @param $nrbeneficiom         Número do benefício do histórico (opcional)
     * @param $nrtpbeneficio        Número do tipo do benefício do histórico (opcional)
     * @param $nrestrutlegal        Estrutura legal do vínculo do histórico (opcional)
     * @param $nrestrutgeren        Estrutura gerencial do vínculo do histórico (opcional)
     * @param $lista_nrvinculom     Lista de vínculos - Formato: '_#####_#####_' (opcional)
     * @return array FpaItecalcfolha
     * @throws \Exception
     */
    public function getItensHistoricoBeneficio($nreventom, $nrorg, $dtmescompetenc, $nrcalculobenef, $nrbeneficiom, $nrtpbeneficio, $nrestrutlegal, $nrestrutgeren, $lista_nrvinculom) {
        try {
            $params = array(
                'P_NREVENTOM'        => $nreventom,
                'P_NRORG'            => $nrorg,
                'P_NRCALCULOFOLHA'   => $nrcalculobenef,
                'P_DTMESCOMPETENC'   => $dtmescompetenc,
                'P_NRBENEFICIOM'     => $nrbeneficiom,
                'P_NRTPBENEFICIO'    => $nrtpbeneficio,
                'P_NRESTRUTLEGAL'    => $nrestrutlegal,
                'P_NRESTRUTGEREN'    => $nrestrutgeren,
                'P_LISTA_NRVINCULOM' => $lista_nrvinculom
            );

            $condTpBeneficio = null;
            $condEstrutura = null;

            if($nrtpbeneficio){
                $condTpBeneficio = "JOIN VAB_BENEFICIOH BH ON BH.NRORG = :P_NRORG
                                                                      AND (BH.NRTPBENEFICIO = :P_NRTPBENEFICIO OR :P_NRTPBENEFICIO IS NULL)
                                                                      AND BH.TPPERIODOBENEF = 'DIARIO'
                                                                      AND BH.NRBENEFICIOM = HB.NRBENEFICIOM
                                                                      AND BH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH
                                                                                                WHERE NRORG = BH.NRORG
                                                                                                  AND NRBENEFICIOM = BH.NRBENEFICIOM
                                                                                                  AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
            }

            if($nrestrutlegal || $nrestrutgeren){
                $condEstrutura = "JOIN GPE_VINCULOH VH ON VH.NRORG = :P_NRORG
                                                                      AND (VH.NRESTRUTLEGAL = :P_NRESTRUTLEGAL OR :P_NRESTRUTLEGAL IS NULL)
                                                                      AND (VH.NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL)
                                                                      AND VH.NRVINCULOM = HB.NRVINCULOM
                                                                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                                                                WHERE NRORG = VH.NRORG
                                                                                                  AND NRVINCULOM = VH.NRVINCULOM
                                                                                                  AND DTMESCOMPETENC <= :P_DTMESCOMPETENC)";
            }

            $where = "(NREVENTOM = :P_NREVENTOM OR :P_NREVENTOM IS NULL)
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA
                AND NRORG = :P_NRORG
                AND (INSTR(:P_LISTA_NRVINCULOM, '_'||NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A')
                AND NRVINCULOM IN (SELECT HB.NRVINCULOM
                                     FROM VAB_HISTBENEFVINC HB
                                     $condTpBeneficio
                                     $condEstrutura
                                    WHERE HB.NRORG = :P_NRORG
                                      AND (HB.NRBENEFICIOM = :P_NRBENEFICIOM OR :P_NRBENEFICIOM IS NULL)
                                      AND (INSTR(:P_LISTA_NRVINCULOM, '_'||HB.NRVINCULOM||'_') > 0 OR :P_LISTA_NRVINCULOM = 'A'))";

            $itens = FpaItecalcfolha::getObjectsByWhere($where, $params);
            return $itens;
        } catch (\Exception $e) {
            throw $e;
        }
    }
} 
